# Generated by Django 3.2.10 on 2022-01-03 05:13

from django.db import migrations, models
import django.db.models.deletion
import wagtail.core.blocks
import wagtail.core.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TestPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.page')),
                ('body', wagtail.core.fields.StreamField([('text', wagtail.core.blocks.StructBlock([('body', wagtail.core.blocks.RichTextBlock(required=False))], template='text_block.html')), ('section', wagtail.core.blocks.StreamBlock([('text', wagtail.core.blocks.StructBlock([('body', wagtail.core.blocks.RichTextBlock(required=False))], template='text_block.html'))])), ('list', wagtail.core.blocks.StructBlock([('title', wagtail.core.blocks.TextBlock(required=False)), ('items', wagtail.core.blocks.StreamBlock([('item', wagtail.core.blocks.StructBlock([('body', wagtail.core.blocks.RichTextBlock(required=False))], template='text_block.html'))]))]))], blank=True, null=True)),
            ],
            bases=('wagtailcore.page',),
        ),
    ]
